# Tests by default. No tests: rpm -ba --define 'dotests 0' *.spec
%{!?dotests: %define dotests 0}

%global __tar /opt/freeware/bin/tar 
%global name abseil-cpp
%global version	20240722.0
%global release	1
%global _libdir64 %{_prefix}/lib64
%global sover 2407.0.0

Name:           %{name}
Version:        %{version}
Release:	1
Summary:        Abseil Common Libraries (C++)
License:        Apache-2.0 
URL:            https://abseil.io
Source0:	https://github.com/abseil/abseil-cpp/archive/refs/tags/%{version}.tar.gz
Source1:        abseil-cpp-20230802-libs.tar.gz

Patch1: abseil-cpp-20230802.0-strerror-test.patch
Patch2: abseil-cpp-20240722.0-test-segfault.patch

BuildRequires:	gcc >= 10.3.0
BuildRequires:	gcc-c++ >= 10.3.0
BuildRequires:	cmake >= 3.30.3
BuildRequires:	gmock-devel >= 1.15.2
BuildRequires:	gtest-devel >= 1.15.2
BuildRequires:	findutils >= 4.9.0
BuildRequires:	sed >= 4.8
BuildRequires:	coreutils >= 9.5


%description
Abseil is an open-source collection of C++ library code designed to augment
the C++ standard library. The Abseil library code is collected from
Google's own C++ code base, has been extensively tested and used in
production, and is the same code we depend on in our daily coding lives.

In some cases, Abseil provides pieces missing from the C++ standard; in
others, Abseil provides alternatives to the standard for special needs we've
found through usage in the Google code base. We denote those cases clearly
within the library code we provide you.

Abseil is not meant to be a competitor to the standard library; we've just
found that many of these utilities serve a purpose within our code base,
and we now want to provide those resources to the C++ community as a whole.

%package devel
Summary:        Development files for %{name}
Requires:       %{name} = %{version}-%{release}
Requires:       %{name}-testing = %{version}-%{release}

%description devel
Development headers for %{name}.


%package testing
Summary:        Libraries needed for running tests on the installed %{name}
Requires:       %{name} = %{version}-%{release}
Requires:	gmock >= 1.15.2
Requires:	gtest >= 1.15.2

%description testing
%{summary}.

%prep
%setup -n  %{name}-%{version}
%patch1 -p1
%patch2 -p1

rm -rf   /tmp/%{name}-%{version}-32bit
cp -pr . /tmp/%{name}-%{version}-32bit
rm -fr *
mv       /tmp/%{name}-%{version}-32bit 32bit
cp -pr 32bit 64bit

%build
#64bit Build
cd 64bit
export OBJECT_MODE=64
export CC="%{__cc__} "
export CFLAGS="-maix64 -pthread -I/opt/freeware/include -fPIC -O2 -I/usr/include"
export CXX="%{__cxx__} "
export CXXFLAGS="-maix64 -pthread -I/opt/freeware/include -fPIC -O2 -I/usr/include"
export LDFLAGS="-Wl,-bbigtoc  -L/opt/freeware/lib/pthread -L/opt/freeware/lib64 -L/opt/freeware/lib -L/usr/lib"

cmake . -B build_64 \
  -DABSL_USE_EXTERNAL_GOOGLETEST:BOOL=ON \
  -DABSL_FIND_GOOGLETEST:BOOL=ON \
  -DABSL_BUILD_TEST_HELPERS=ON \
  -DABSL_BUILD_TESTING:BOOL=ON \
  -DBUILD_SHARED_LIBS=ON \
  -DCMAKE_INSTALL_LIBDIR=lib64 \
  -DABSL_ENABLE_INSTALL=ON \
  -DCMAKE_AIX_SHARED_LIBRARY_ARCHIVE=ON \
  -DCMAKE_INSTALL_PREFIX=%{_prefix}

cmake --build build_64  -j8

#32bit Build
cd ../32bit
export OBJECT_MODE=32
export CC="%{__cc__} "
export CFLAGS="-maix32 -pthread -I/opt/freeware/include -fPIC -O2 -I/usr/include"
export CXX="%{__cxx__} "
export CXXFLAGS="-maix32 -pthread -I/opt/freeware/include -fPIC -O2 -I/usr/include"
export LDFLAGS="-Wl,-bbigtoc -latomic -L/opt/freeware/lib/pthread  -L/opt/freeware/lib -L/usr/lib"

cmake . -B build_32 \
  -DABSL_USE_EXTERNAL_GOOGLETEST:BOOL=ON \
  -DABSL_FIND_GOOGLETEST:BOOL=ON \
  -DABSL_BUILD_TEST_HELPERS=ON \
  -DABSL_BUILD_TESTING:BOOL=ON \
  -DBUILD_SHARED_LIBS=ON \
  -DCMAKE_INSTALL_LIBDIR=lib \
  -DABSL_ENABLE_INSTALL=ON \
  -DCMAKE_AIX_SHARED_LIBRARY_ARCHIVE=ON \
  -DCMAKE_INSTALL_PREFIX=%{_prefix}

cmake --build build_32 -j8

%install
export DESTDIR=$RPM_BUILD_ROOT
cd 64bit
cmake  --install build_64

cd ../32bit
export OBJECT_MODE=32
cmake  --install build_32
#archives 64bit shared objects into 32bit archivesldroot}%{_libdir64}/
cd ${RPM_BUILD_ROOT}/%{_libdir64}/
ABSL_LIB_LISTS=`ls *.a`
for file in $ABSL_LIB_LISTS
do
        /usr/bin/ar -X64 -xv $file
        so_file="${file%.*}"
        /usr/bin/ar -X64 -q %{buildroot}%{_libdir}/$file $so_file.so.%{sover}
done
cd -

#archive previous release so
cd ${RPM_BUILD_ROOT}/%{_libdir}/
tar -xvf %{SOURCE1} 
cd ${RPM_BUILD_ROOT}/%{_libdir}/abseil-cpp-20230802-libs/64bit
FILE_LIST=`ls`
cd -
for file in $FILE_LIST
do
	file_name=`echo $file | cut -d '.' -f 1`
	if [[ -f ${RPM_BUILD_ROOT}/%{_libdir}//$file_name.a ]]; then
        /usr/bin/strip -X64 -e ${RPM_BUILD_ROOT}/%{_libdir}/abseil-cpp-20230802-libs/64bit/$file
		/usr/bin/ar -X64 -q ${RPM_BUILD_ROOT}/%{_libdir}/$file_name.a  ${RPM_BUILD_ROOT}/%{_libdir}/abseil-cpp-20230802-libs/64bit/$file
        /usr/bin/strip -X32 -e ${RPM_BUILD_ROOT}/%{_libdir}/abseil-cpp-20230802-libs/32bit/$file
		/usr/bin/ar -X32 -q ${RPM_BUILD_ROOT}/%{_libdir}/$file_name.a  ${RPM_BUILD_ROOT}/%{_libdir}/abseil-cpp-20230802-libs/32bit/$file
	fi
done
#libabsl_flags is removed in 20240722.0, but to support previous version , we need to copy this seperately.
/usr/bin/strip -X32 -e ${RPM_BUILD_ROOT}/%{_libdir}/abseil-cpp-20230802-libs/32bit/libabsl_flags.so.2308.0.0
/usr/bin/ar -X32 -q ${RPM_BUILD_ROOT}/%{_libdir}/libabsl_flags.a ${RPM_BUILD_ROOT}/%{_libdir}/abseil-cpp-20230802-libs/32bit/libabsl_flags.so.2308.0.0
/usr/bin/strip -X64 -e ${RPM_BUILD_ROOT}/%{_libdir}/abseil-cpp-20230802-libs/64bit/libabsl_flags.so.2308.0.0
/usr/bin/ar -X64 -q ${RPM_BUILD_ROOT}/%{_libdir}/libabsl_flags.a ${RPM_BUILD_ROOT}/%{_libdir}/abseil-cpp-20230802-libs/64bit/libabsl_flags.so.2308.0.0

#create link for 64bit lib which refers to 32bit shared archives
cd $RPM_BUILD_ROOT%{_prefix}/lib64
FILE_LIST=`ls *.a`
for file in $FILE_LIST
do
    rm $file
    ln -s ../lib/$file $file
done
ln -s ../lib/libabsl_flags.a libabsl_flags.a

%check
cd 64bit/build_64
ctest || true
cd ../../32bit/build_32
ctest || true

%files
%defattr(-, root, system,-)
%doc 64bit/FAQ.md 64bit/README.md 64bit/UPGRADES.md 64bit/LICENSE
#64bit
%{_libdir64}/libabsl_flags.a
%{_libdir64}/libabsl_bad_any_cast_impl.a
%{_libdir64}/libabsl_bad_optional_access.a
%{_libdir64}/libabsl_bad_variant_access.a
%{_libdir64}/libabsl_base.a
%{_libdir64}/libabsl_city.a
%{_libdir64}/libabsl_civil_time.a
%{_libdir64}/libabsl_cord.a
%{_libdir64}/libabsl_cord_internal.a
%{_libdir64}/libabsl_cordz_functions.a
%{_libdir64}/libabsl_cordz_handle.a
%{_libdir64}/libabsl_cordz_info.a
%{_libdir64}/libabsl_cordz_sample_token.a
%{_libdir64}/libabsl_crc32c.a
%{_libdir64}/libabsl_crc_cord_state.a
%{_libdir64}/libabsl_crc_cpu_detect.a
%{_libdir64}/libabsl_crc_internal.a
%{_libdir64}/libabsl_debugging_internal.a
%{_libdir64}/libabsl_decode_rust_punycode.a
%{_libdir64}/libabsl_demangle_internal.a
%{_libdir64}/libabsl_demangle_rust.a
%{_libdir64}/libabsl_die_if_null.a
%{_libdir64}/libabsl_examine_stack.a
%{_libdir64}/libabsl_exponential_biased.a
%{_libdir64}/libabsl_failure_signal_handler.a
%{_libdir64}/libabsl_flags_commandlineflag.a
%{_libdir64}/libabsl_flags_commandlineflag_internal.a
%{_libdir64}/libabsl_flags_config.a
%{_libdir64}/libabsl_flags_internal.a
%{_libdir64}/libabsl_flags_marshalling.a
%{_libdir64}/libabsl_flags_parse.a
%{_libdir64}/libabsl_flags_program_name.a
%{_libdir64}/libabsl_flags_private_handle_accessor.a
%{_libdir64}/libabsl_flags_reflection.a
%{_libdir64}/libabsl_flags_usage.a
%{_libdir64}/libabsl_flags_usage_internal.a
%{_libdir64}/libabsl_graphcycles_internal.a
%{_libdir64}/libabsl_hash.a
%{_libdir64}/libabsl_hashtablez_sampler.a
%{_libdir64}/libabsl_int128.a
%{_libdir64}/libabsl_kernel_timeout_internal.a
%{_libdir64}/libabsl_leak_check.a
%{_libdir64}/libabsl_log_entry.a
%{_libdir64}/libabsl_log_flags.a
%{_libdir64}/libabsl_log_globals.a
%{_libdir64}/libabsl_log_initialize.a
%{_libdir64}/libabsl_log_internal_check_op.a
%{_libdir64}/libabsl_log_internal_conditions.a
%{_libdir64}/libabsl_log_internal_fnmatch.a
%{_libdir64}/libabsl_log_internal_format.a
%{_libdir64}/libabsl_log_internal_globals.a
%{_libdir64}/libabsl_log_internal_log_sink_set.a
%{_libdir64}/libabsl_log_internal_message.a
%{_libdir64}/libabsl_log_internal_nullguard.a
%{_libdir64}/libabsl_log_internal_proto.a
%{_libdir64}/libabsl_log_severity.a
%{_libdir64}/libabsl_log_sink.a
%{_libdir64}/libabsl_low_level_hash.a
%{_libdir64}/libabsl_malloc_internal.a
%{_libdir64}/libabsl_periodic_sampler.a
%{_libdir64}/libabsl_poison.a
%{_libdir64}/libabsl_random_distributions.a
%{_libdir64}/libabsl_random_internal_distribution_test_util.a
%{_libdir64}/libabsl_random_internal_platform.a
%{_libdir64}/libabsl_random_internal_pool_urbg.a
%{_libdir64}/libabsl_random_internal_randen.a
%{_libdir64}/libabsl_random_internal_randen_hwaes.a
%{_libdir64}/libabsl_random_internal_randen_hwaes_impl.a
%{_libdir64}/libabsl_random_internal_randen_slow.a
%{_libdir64}/libabsl_random_internal_seed_material.a
%{_libdir64}/libabsl_random_seed_gen_exception.a
%{_libdir64}/libabsl_random_seed_sequences.a
%{_libdir64}/libabsl_raw_hash_set.a
%{_libdir64}/libabsl_raw_logging_internal.a
%{_libdir64}/libabsl_scoped_set_env.a
%{_libdir64}/libabsl_spinlock_wait.a
%{_libdir64}/libabsl_stacktrace.a
%{_libdir64}/libabsl_status.a
%{_libdir64}/libabsl_statusor.a
%{_libdir64}/libabsl_str_format_internal.a
%{_libdir64}/libabsl_strerror.a
%{_libdir64}/libabsl_strings.a
%{_libdir64}/libabsl_strings_internal.a
%{_libdir64}/libabsl_string_view.a
%{_libdir64}/libabsl_symbolize.a
%{_libdir64}/libabsl_synchronization.a
%{_libdir64}/libabsl_throw_delegate.a
%{_libdir64}/libabsl_time.a
%{_libdir64}/libabsl_time_zone.a
%{_libdir64}/libabsl_utf8_for_code_point.a
%{_libdir64}/libabsl_vlog_config_internal.a

#32bit
%{_libdir}/libabsl_flags.a
%{_libdir}/libabsl_bad_any_cast_impl.a
%{_libdir}/libabsl_bad_optional_access.a
%{_libdir}/libabsl_bad_variant_access.a
%{_libdir}/libabsl_base.a
%{_libdir}/libabsl_city.a
%{_libdir}/libabsl_civil_time.a
%{_libdir}/libabsl_cord.a
%{_libdir}/libabsl_cord_internal.a
%{_libdir}/libabsl_cordz_functions.a
%{_libdir}/libabsl_cordz_handle.a
%{_libdir}/libabsl_cordz_info.a
%{_libdir}/libabsl_cordz_sample_token.a
%{_libdir}/libabsl_crc32c.a
%{_libdir}/libabsl_crc_cord_state.a
%{_libdir}/libabsl_crc_cpu_detect.a
%{_libdir}/libabsl_crc_internal.a
%{_libdir}/libabsl_debugging_internal.a
%{_libdir}/libabsl_decode_rust_punycode.a
%{_libdir}/libabsl_demangle_internal.a
%{_libdir}/libabsl_demangle_rust.a
%{_libdir}/libabsl_die_if_null.a
%{_libdir}/libabsl_examine_stack.a
%{_libdir}/libabsl_exponential_biased.a
%{_libdir}/libabsl_failure_signal_handler.a
%{_libdir}/libabsl_flags_commandlineflag.a
%{_libdir}/libabsl_flags_commandlineflag_internal.a
%{_libdir}/libabsl_flags_private_handle_accessor.a
%{_libdir}/libabsl_flags_config.a
%{_libdir}/libabsl_flags_internal.a
%{_libdir}/libabsl_flags_marshalling.a
%{_libdir}/libabsl_flags_parse.a
%{_libdir}/libabsl_flags_program_name.a
%{_libdir}/libabsl_flags_reflection.a
%{_libdir}/libabsl_flags_usage.a
%{_libdir}/libabsl_flags_usage_internal.a
%{_libdir}/libabsl_graphcycles_internal.a
%{_libdir}/libabsl_hash.a
%{_libdir}/libabsl_hashtablez_sampler.a
%{_libdir}/libabsl_int128.a
%{_libdir}/libabsl_kernel_timeout_internal.a
%{_libdir}/libabsl_leak_check.a
%{_libdir}/libabsl_log_entry.a
%{_libdir}/libabsl_log_flags.a
%{_libdir}/libabsl_log_globals.a
%{_libdir}/libabsl_log_initialize.a
%{_libdir}/libabsl_log_internal_check_op.a
%{_libdir}/libabsl_log_internal_conditions.a
%{_libdir}/libabsl_log_internal_fnmatch.a
%{_libdir}/libabsl_log_internal_format.a
%{_libdir}/libabsl_log_internal_globals.a
%{_libdir}/libabsl_log_internal_log_sink_set.a
%{_libdir}/libabsl_log_internal_message.a
%{_libdir}/libabsl_log_internal_nullguard.a
%{_libdir}/libabsl_log_internal_proto.a
%{_libdir}/libabsl_log_severity.a
%{_libdir}/libabsl_log_sink.a
%{_libdir}/libabsl_low_level_hash.a
%{_libdir}/libabsl_malloc_internal.a
%{_libdir}/libabsl_periodic_sampler.a
%{_libdir}/libabsl_poison.a
%{_libdir}/libabsl_random_distributions.a
%{_libdir}/libabsl_random_internal_distribution_test_util.a
%{_libdir}/libabsl_random_internal_platform.a
%{_libdir}/libabsl_random_internal_pool_urbg.a
%{_libdir}/libabsl_random_internal_randen.a
%{_libdir}/libabsl_random_internal_randen_hwaes.a
%{_libdir}/libabsl_random_internal_randen_hwaes_impl.a
%{_libdir}/libabsl_random_internal_randen_slow.a
%{_libdir}/libabsl_random_internal_seed_material.a
%{_libdir}/libabsl_random_seed_gen_exception.a
%{_libdir}/libabsl_random_seed_sequences.a
%{_libdir}/libabsl_raw_hash_set.a
%{_libdir}/libabsl_raw_logging_internal.a
%{_libdir}/libabsl_scoped_set_env.a
%{_libdir}/libabsl_spinlock_wait.a
%{_libdir}/libabsl_stacktrace.a
%{_libdir}/libabsl_status.a
%{_libdir}/libabsl_statusor.a
%{_libdir}/libabsl_str_format_internal.a
%{_libdir}/libabsl_strerror.a
%{_libdir}/libabsl_strings.a
%{_libdir}/libabsl_strings_internal.a
%{_libdir}/libabsl_string_view.a
%{_libdir}/libabsl_symbolize.a
%{_libdir}/libabsl_synchronization.a
%{_libdir}/libabsl_throw_delegate.a
%{_libdir}/libabsl_time.a
%{_libdir}/libabsl_time_zone.a
%{_libdir}/libabsl_utf8_for_code_point.a
%{_libdir}/libabsl_vlog_config_internal.a


%files testing
%defattr(-, root, system,-)
# TESTONLY libraries (that are actually installed):
#64bit
# absl/base/CMakeLists.txt
%{_libdir64}/libabsl_exception_safety_testing.a
%{_libdir64}/libabsl_atomic_hook_test_helper.a
%{_libdir64}/libabsl_spinlock_test_common.a
# absl/container/CMakeLists.txt
%{_libdir64}/libabsl_test_instance_tracker.a
%{_libdir64}/libabsl_hash_generator_testing.a
# absl/debugging/CMakeLists.txt
%{_libdir64}/libabsl_stack_consumption.a
# absl/log/CMakeLists.txt
%{_libdir64}/libabsl_log_internal_test_actions.a
%{_libdir64}/libabsl_log_internal_test_helpers.a
%{_libdir64}/libabsl_log_internal_test_matchers.a
%{_libdir64}/libabsl_scoped_mock_log.a
# absl/status/CMakeLists.txt
%{_libdir64}/libabsl_status_matchers.a
# absl/strings/CMakeLists.txt
%{_libdir64}/libabsl_pow10_helper.a
# absl/synchronization/CMakeLists.txt
%{_libdir64}/libabsl_per_thread_sem_test_common.a
# absl/time/CMakeLists.txt
%{_libdir64}/libabsl_time_internal_test_util.a

#32bit
# absl/base/CMakeLists.txt
%{_libdir}/libabsl_exception_safety_testing.a
%{_libdir}/libabsl_atomic_hook_test_helper.a
%{_libdir}/libabsl_spinlock_test_common.a
# absl/container/CMakeLists.txt
%{_libdir}/libabsl_test_instance_tracker.a
%{_libdir}/libabsl_hash_generator_testing.a
# absl/debugging/CMakeLists.txt
%{_libdir}/libabsl_stack_consumption.a
# absl/log/CMakeLists.txt
%{_libdir}/libabsl_log_internal_test_actions.a
%{_libdir}/libabsl_log_internal_test_helpers.a
%{_libdir}/libabsl_log_internal_test_matchers.a
%{_libdir}/libabsl_scoped_mock_log.a
# absl/status/CMakeLists.txt
%{_libdir}/libabsl_status_matchers.a
# absl/strings/CMakeLists.txt
%{_libdir}/libabsl_pow10_helper.a
# absl/synchronization/CMakeLists.txt
%{_libdir}/libabsl_per_thread_sem_test_common.a
# absl/time/CMakeLists.txt
%{_libdir}/libabsl_time_internal_test_util.a


%files devel
%defattr(-, root, system,-)
%{_includedir}/absl
%{_libdir64}/pkgconfig/absl_*.pc
%{_libdir64}/cmake/absl
%{_libdir}/pkgconfig/absl_*.pc
%{_libdir}/cmake/absl

%changelog
* Wed Sep 04 2024 Ranjit Ranjan <ranjit.ranjan@ibm.com> - 20240722.0-1 
- Update to version 20240722.0

* Tue Aug 22 2023 Ranjit Ranjan <ranjit.ranjan@ibm.com> - 20230802.0-1
- Port on AIX.
